/*
 * Print and Close icons from Silk icon set 1.3
 * _________________________________________
 * Mark James
 * http://www.famfamfam.com/lab/icons/silk/
 */

html.x-ux-grid-printer,
.x-ux-grid-printer div,
.x-ux-grid-printer dl,
.x-ux-grid-printer dt,
.x-ux-grid-printer dd,
.x-ux-grid-printer ul,
.x-ux-grid-printer ol,
.x-ux-grid-printer li,
.x-ux-grid-printer h1,
.x-ux-grid-printer h2,
.x-ux-grid-printer h3,
.x-ux-grid-printer h4,
.x-ux-grid-printer h5,
.x-ux-grid-printer h6,
.x-ux-grid-printer pre,
.x-ux-grid-printer form,
.x-ux-grid-printer fieldset,
.x-ux-grid-printer input,
.x-ux-grid-printer p,
.x-ux-grid-printer blockquote,
.x-ux-grid-printer th,
.x-ux-grid-printer td {
  margin: 0;
  padding: 0;
}

html.x-ux-grid-printer,
.x-ux-grid-printer img,
.x-ux-grid-printer body {
  border: 0;
}

.x-ux-grid-printer address,
.x-ux-grid-printer caption,
.x-ux-grid-printer cite,
.x-ux-grid-printer code,
.x-ux-grid-printer dfn,
.x-ux-grid-printer em,
.x-ux-grid-printer strong,
.x-ux-grid-printer th,
.x-ux-grid-printer var {
  font-style: normal;
  font-weight: normal;
}

.x-ux-grid-printer ol,
.x-ux-grid-printer ul {
  list-style: none;
}

.x-ux-grid-printer caption,
.x-ux-grid-printer th {
  text-align: left;
}

.x-ux-grid-printer h1,
.x-ux-grid-printer h2,
.x-ux-grid-printer h3,
.x-ux-grid-printer h4,
.x-ux-grid-printer h5,
.x-ux-grid-printer h6 {
  font-size: 100%;
}

.x-ux-grid-printer q:before,
.x-ux-grid-printer q:after {
  content: "";
}

.x-ux-grid-printer .x-ux-grid-printer-links {
  padding: 0 0 20px 0;
}

.x-ux-grid-printer a.x-ux-grid-printer-linkprint {
  padding: 0 0 0 18px;
  margin: 0 5px;
  background: url(../img/Printer/printer.png) no-repeat bottom left !important;
}

.x-ux-grid-printer a.x-ux-grid-printer-linkclose {
  padding: 0 0 0 18px;
  margin: 0 5px;
  background: url(../img/Printer/cross.png) no-repeat bottom left !important;
}

.x-ux-grid-printer table {
  width: 100%;
  text-align: left;
  font-size: 11px;
  font-family: arial;
  border-collapse: collapse;
}

.x-ux-grid-printer table th {
  font-weight: bold;
  padding: 4px 3px 4px 5px;
  border: 1px solid #d0d0d0;
  border-left-color: #eee;
  background-color: #ededed;
}

.x-ux-grid-printer table tr.odd {
  background-color: #ffffff;
}

.x-ux-grid-printer table tr.even {
  background-color: #f9f9f9;
}

.x-ux-grid-printer table td {
  padding: 4px 3px 4px 5px;
  border-style: none solid solid;
  border-width: 1px;
  border-color: #ededed;
}

.x-tree-elbow-img,
.x-tree-elbow-line {
  width: 10px;
}

@media print {
  .x-ux-grid-printer body {
    margin: 0;
    padding: 0;
  }
  .x-ux-grid-printer div.x-ux-grid-printer-noprint {
    display: none;
  }
}
.group-header {
  border-bottom: 2px solid black;
  font-size: 14px;
}
