.x-boxselect.x-form-text {
  height: auto;
  cursor: text;
  padding: 0;
  overflow-y: auto;
}
.x-boxselect ul.x-boxselect-list {
  padding: 1px 3px;
  margin-top: 0px;
  margin-bottom: 0px;
}
ul.x-boxselect-list.x-boxselect-singleselect {
  white-space: nowrap;
  overflow: hidden;
}
.x-boxselect input {
  border: none;
  background: none;
  line-height: 18px;
  height: 20px;
  width: 100%;
}
.x-boxselect-input,
.x-boxselect-item {
  display: inline-block;
  position: relative;
  *display: inline; /* IE7 */
  transform: scale(1); /* IE */
}
.x-boxselect-stacked .x-boxselect-item {
  display: block;
}
.x-boxselect-item {
  -moz-border-radius: 6px;
  -webkit-border-radius: 6px;
  border-radius: 6px;
  o-border-radius: 6px;
  khtml-border-radius: 6px;
  border: 1px solid #cad8f3;
  padding: 0px 1px 0px 5px !important;
  margin: 1px 1px 1px 0;
  background-color: #dee7f8;
  cursor: default;
}
.x-field:not(.x-item-disabled) .x-boxselect-item:hover {
  background: #bbcef1;
  border: 1px solid #6d95e0;
}
.x-field:not(.x-item-disabled) .x-boxselect-item.selected {
  border: 1px solid #316ad3 !important;
  background: #658fdc !important;
}
.x-boxselect-item-text {
  line-height: 16px;
  padding-right: 20px !important;
}
.x-boxselect-item-close {
  cursor: pointer;
}
.x-boxselect-hideselections .x-boundlist-selected {
  display: none;
}

.x-form-not-trigger {
  background-image: url(../img/DiracBoxSelect/not-trigger.gif);
}

.x-form-notsel-trigger {
  background-image: url(../img/DiracBoxSelect/not-trigger-sel.gif);
}
