/* Pictos Custom Font
* Copyright (C) 2010-2012 Drew Wilson
* http://www.pictos.cc/server    -    http://drewwilson.com
*
* The fonts included in this document and used on this website are subject to the
* End User License Agreement entered into by the website owner. The fonts are protected
* under domestic and international trademark and copyright law. All parties other than
* the owner of this website are prohibited from using, modifying, reverse engineering,
* duplicating, or distributing the fonts in this document.
* These fonts are NOT free. It is illegal to use these fonts without paying for a license.
*
*/
@font-face {
  font-family: 'Pictos';
  src: url('pictos.eot');
  src: url('pictos.eot?#iefix') format('embedded-opentype'),
  		url('pictos.woff') format('woff'),
  		url('pictos.ttf') format('truetype');
  font-weight: normal;
  font-style: normal;
}