.dirac-icon-submit {
  background-image: url(../img/icons/common/submit.gif) !important;
  background-color: transparent;
}

.dirac-icon-restart {
  background-image: url(../img/icons/common/restart.png) !important;
  background-color: transparent;
}

.dirac-icon-start {
  background-image: url(../img/icons/common/submit.gif) !important;
  background-color: transparent;
}

.dirac-icon-revert {
  background-image: url(../img/icons/common/revert.png) !important;
  background-color: transparent;
}

.dirac-icon-stop {
  background-image: url(../img/icons/common/stop.gif) !important;
  background-color: transparent;
}

.dirac-icon-log {
  background-image: url(../img/icons/common/log.gif) !important;
  background-color: transparent;
}

.dirac-icon-error {
  background-image: url(../img/icons/common/error.png) !important;
  background-color: transparent;
}

.dirac-icon-mail {
  background-image: url(../img/icons/common/mail.png) !important;
  background-color: transparent;
}

.dirac-icon-refresh {
  background-image: url(../img/icons/common/refresh.gif) !important;
  background-color: transparent;
}

.dirac-icon-save {
  background-image: url(../img/icons/common/save.png) !important;
  background-color: transparent;
}

.dirac-icon-delete {
  background-image: url(../img/icons/common/delete.gif) !important;
  background-color: transparent;
}

.dirac-icon-reset {
  background-image: url(../img/icons/common/reset.gif) !important;
  background-color: transparent;
}

.dirac-icon-reschedule {
  background-image: url(../img/icons/common/reschedule.gif) !important;
  background-color: transparent;
}

.dirac-icon-plus {
  background-image: url(../img/icons/common/plus.png) !important;
  background-color: transparent;
}

.dirac-icon-download {
  background-image: url(../img/icons/common/download.png) !important;
  background-color: transparent;
}

.dirac-icon-upload {
  background-image: url(../img/icons/common/upload.png) !important;
  background-color: transparent;
}

.dirac-icon-link {
  background-image: url(../img/icons/common/link.gif) !important;
  background-color: transparent;
}

.dirac-icon-share {
  background-image: url(../img/icons/common/share.png) !important;
  background-color: transparent;
}

.dirac-icon-state {
  background-image: url(../img/icons/common/state.png) !important;
  background-color: transparent;
}

.dirac-icon-kill {
  background-image: url(../img/icons/common/kill.gif) !important;
  background-color: transparent;
}

.dirac-icon-action {
  background-image: url(../img/icons/common/action.png) !important;
  background-color: transparent;
}

.dirac-icon-list {
  background-image: url(../img/icons/common/list.png) !important;
  background-color: transparent;
}

.dirac-icon-pie {
  background-image: url(../img/icons/common/pie.png) !important;
  background-color: transparent;
}

.dirac-icon-text {
  background-image: url(../img/icons/common/text.png) !important;
  background-color: transparent;
}

.dirac-icon-proportional {
  background-image: url(../img/icons/common/proportional.png) !important;
  background-color: transparent;
}

.dirac-icon-stretch {
  background-image: url(../img/icons/common/stretch.png) !important;
  background-color: transparent;
}

.dirac-icon-edit {
  background-image: url(../img/icons/common/edit.png) !important;
  background-color: transparent;
}

.dirac-icon-private {
  background-image: url(../img/icons/common/private-icon.png) !important;
  background-color: transparent;
}

.dirac-icon-new-folder {
  background-image: url(../img/icons/common/new-folder.png) !important;
  background-color: transparent;
}

.dirac-icon-private-folder {
  background-image: url(../img/icons/common/private-folder.png) !important;
  background-color: transparent;
}

.dirac-icon-tab-theme {
  background-image: url(../img/icons/common/tabTheme.png) !important;
  background-color: transparent;
}

.dirac-icon-presenter-theme {
  background-image: url(../img/icons/common/presenterTheme.png) !important;
  background-color: transparent;
}

.dirac-icon-print {
  background-image: url("../img/icons/system/printer.png") !important;
  background-color: transparent;
}

.dirac-icon-update {
  background-image: url(../img/icons/common/update.png) !important;
  background-color: transparent;
}

.dirac-icon-settings {
  background-image: url(../img/icons/fam/cog.png) !important;
  background-color: transparent;
}
