.accp-reschedule-icon {
  background-image: url(../img/PlotView/reschedule.gif) !important;
  background-color: transparent;
}

.accp-kill-icon {
  background-image: url(../img/PlotView/close.gif) !important;
  background-color: transparent;
}

.accp-delete-icon {
  background-image: url(../img/PlotView/delete.gif) !important;
  background-color: transparent;
}

.accp-refresh-icon {
  background-image: url(../img/PlotView/refresh.gif) !important;
  background-color: transparent;
}

.accp-submit-icon {
  background-image: url(../img/PlotView/submit.gif) !important;
  background-color: transparent;
}

.accp-reset-icon {
  background-image: url(../img/PlotView/reset.gif) !important;
  background-color: transparent;
}

.accp-reset-button-icon {
  background-image: url(../img/PlotView/resetButton.gif) !important;
  background-color: transparent;
}

.accp-action-gif-icon {
  background-image: url(../img/PlotView/action.gif) !important;
  background-color: transparent;
}

.accp-addfile-gif-icon {
  background-image: url(../img/PlotView/addfile.gif) !important;
  background-color: transparent;
}

.accp-textbox-help-window textarea.x-form-field {
  font-family: Courier, monospace !important;
}

.accp-child-window-gif-focus {
  background-image: url(../img/PlotView/done.gif) !important;
  background-color: transparent;
}

.accp-child-window-gif-notfocus {
  background-image: url(../img/PlotView/failed.gif) !important;
  background-color: transparent;
}

#images-view .x-panel-body {
  background: white;
  font: 11px Arial, Helvetica, sans-serif;
}
#images-view .thumb {
  background: #dddddd;
  padding: 3px;
  padding-bottom: 0;
}

.x-quirks #images-view .thumb {
  padding-bottom: 3px;
}

#images-view .thumb img {
  height: 60px;
  width: 80px;
}
#images-view .thumb-wrap {
  float: left;
  margin: 4px;
  margin-right: 0;
  padding: 5px;
}
#images-view .thumb-wrap span {
  display: block;
  overflow: hidden;
  text-align: center;
  width: 86px; /* for ie to ensure that the text is centered */
}

#images-view .x-item-over {
  border: 1px solid #dddddd;
  background: #efefef url(over.gif) repeat-x left top;
  padding: 4px;
}

#images-view .x-item-selected {
  background: #eff5fb url(selected.gif) no-repeat right bottom;
  border: 1px solid #99bbe8;
  padding: 4px;
}
#images-view .x-item-selected .thumb {
  background: transparent;
}

#images-view .loading-indicator {
  font-size: 11px;
  background-image: url("../../resources/themes/images/default/grid/loading.gif");
  background-repeat: no-repeat;
  background-position: left;
  padding-left: 20px;
  margin: 10px;
}

.x-view-selector {
  position: absolute;
  left: 0;
  top: 0;
  width: 0;
  border: 1px dotted;
  opacity: 0.5;
  -moz-opacity: 0.5;
  filter: alpha(opacity=50);
  transform: scale(1);
  background-color: #c3daf9;
  border-color: #3399bb;
}
.ext-strict .ext-ie .x-tree .x-panel-bwrap {
  position: relative;
  overflow: hidden;
}

#plots {
  background-color: #fff;
  text-shadow: #fff 0 1px 0;
  position: relative;
  display: block;
  height: auto;
}

#plots div.plot img {
  margin-bottom: 1px;
}

#plots div.plot {
  float: left;
  padding: 8px 17px;
  margin: 5px;
  /*    margin: 10px 0 0 25px;*/
  text-align: center;
  line-height: 14px;
  color: #333;
  font-size: 10px;
  font-family: "Helvetica Neue", sans-serif;
  height: 420px;
  width: 430px;
  overflow: hidden;
  border-top: 1px solid transparent;
  cursor: pointer;
}

.x-ie6 #plots div.plot,
.x-ie7 #plots div.plot,
.x-ie8 #plots div.plot {
  border-top: none;
  padding: 3px 2px;
  margin: 2px;
}

#plots div.plot-hover {
  background-color: #eee;
}

#plots .x-item-selected {
  background-color: #d3e1f1 !important;
}

#plots div.plot strong {
  color: #000;
  display: block;
}

#plots div.plot span {
  color: #999;
}

.my-img {
  border: 1px solid black;
}
