.core-desktop-icon {
  background: url(../img/SelPanel/user-desktop.png) no-repeat !important;
}

.x-grid-tree-node-expanded .x-tree-icon-parent.core-desktop-icon {
  background: url(../img/SelPanel/user-desktop.png) no-repeat !important;
}

.core-application-icon {
  background: url(../img/SelPanel/executable.ico) no-repeat !important;
}

.x-grid-tree-node-expanded .x-tree-icon-parent.core-application-icon {
  background: url(../img/SelPanel/executable.ico) no-repeat !important;
}

/*
.x-grid-tree-node-expanded .x-tree-icon-parent {
    background: url(../img/SelPanel/draw-triangle4.ico) no-repeat center center !    important;
}
*/

.icon-applications-states-all-default {
  background: url(../img/SelPanel/run.ico) no-repeat !important;
  background-color: transparent;
}

.shared-desktop {
  background: url(../img/SelPanel/shared-desktop1.ico) no-repeat !important;
  background-color: transparent;
}

.x-grid-tree-node-expanded .x-tree-icon-parent.shared-desktop {
  background: url(../img/SelPanel/shared-desktop1.ico) no-repeat !important;
}

.my-desktop {
  background: url(../img/SelPanel/home.ico) no-repeat !important;
  background-color: transparent;
}

.x-grid-tree-node-expanded .x-tree-icon-parent.my-desktop {
  background: url(../img/SelPanel/home.ico) no-repeat !important;
}

.list {
  padding-left: 16px;
  padding-bottom: 10px;
}
.list li {
  list-style: square;
  padding: 2px;
}

pre {
  font-size: 11px;
}

@font-face {
  font-family: "Pictos";
  src: url("../pictos/pictos.eot");
  src: url("../pictos/pictos.eot?#iefix") format("embedded-opentype"), url("../pictos/pictos.woff") format("woff"),
    url("../pictos/pictos.ttf") format("truetype");
  font-weight: normal;
  font-style: normal;
}
