.ux-notification-window .x-window-body {
  text-align: center;
  padding: 15px 5px 15px 5px;
}

.ux-notification-icon-information {
  background-image: url("../img/Notification/icon16_info.png");
}

.ux-notification-icon-error {
  background-image: url("../img/Notification/icon16_error.png");
}

.ux-notification-light .x-window-header {
  background-color: transparent;
}

.ux-notification-light {
  background-image: url("../img/Notification/fader.png");
}

.ux-notification-light .x-window-body {
  text-align: center;
  padding: 15px 5px 20px 5px;
  background-color: transparent;
  border: 0px solid white;
}
